%global __provides_exclude_from ^%{_libdir}/lld/.*$
%global __requires_exclude ^libgtest.*$

%global _lto_cflags %{nil}

%global lit_cfg test/%{_arch}.site.cfg.py
%global lit_unit_cfg test/Unit/%{_arch}.site.cfg.py
%global lit_lld_test_cfg_install_path %{_datadir}/lld/lit.lld-test.cfg.py

Summary:		The LLVM Linker
Name:			lld
Version:		17.0.6
Release:		2%{?dist}
License:		NCSA
URL:			http://llvm.org
Source0:		https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}/lld-%{version}.src.tar.xz
Source1:		run-lit-tests
Source2:		lit.lld-test.cfg.py

Patch0001:		0001-Backport-lld-LoongArch-Support-the-R_LARCH_-ADD-SUB-6-relocation-type.patch
Patch0002:		0002-Backport-ELF-RISCV-Implement-emit-relocs-with-relaxation.patch
Patch0003:		0003-Backport-lld-ELF-Support-relax-R_LARCH_ALIGN.patch
Patch0004:		0004-Backport-lld-LoongArch-Support-the-R_LARCH_-ADD-SUB-_ULEB128-relocation-types.patch

Patch3000:		0002-PATCH-lld-Import-compact_unwind_encoding.h-from-libu.patch


%define maj_ver %(echo %{version} | cut -d. -f1)
%define min_ver %(echo %{version} | cut -d. -f2)
%define patch_ver %(echo %{version} | cut -d. -f3)

BuildRequires:		gcc gcc-c++ cmake ninja-build
BuildRequires:		ncurses-devel zlib-devel
BuildRequires:		llvm-devel = %{version}
BuildRequires:		llvm-cmake-utils = %{version}
BuildRequires:		llvm-test = %{version}
BuildRequires:		llvm-googletest = %{version}
BuildRequires:		python3-rpm-macros python3-lit
Requires(post):		%{_sbindir}/update-alternatives
Requires(preun):	%{_sbindir}/update-alternatives
Requires:		lld-libs = %{version}-%{release}

%description
The LLVM project linker.

%package devel
Summary:		Libraries and header files for LLD
Requires:		lld-libs = %{version}-%{release}
Requires:		%{name} = %{version}-%{release}

%description devel
This package contains library and header files needed to develop new native
programs that use the LLD infrastructure.

%package libs
Summary:		LLD shared libraries

%description libs
Shared libraries for LLD.

%package test
Summary:		LLD regression tests
Requires:		%{name} = %{version}-%{release}
Requires:		python3-lit
Requires:		llvm-test(major) = %{maj_ver}
Requires:		lld-libs = %{version}-%{release}

%description test
LLVM regression tests.

%prep
%autosetup -n lld-%{version}.src -p2

%build
%cmake \
	-GNinja \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DLLVM_COMMON_CMAKE_UTILS=%{_datadir}/llvm/cmake \
	-DLLVM_DYLIB_COMPONENTS="all" \
	-DCMAKE_SKIP_RPATH:BOOL=ON \
	-DPYTHON_EXECUTABLE=%{__python3} \
	-DLLVM_INCLUDE_TESTS=ON \
	-DLLVM_MAIN_SRC_DIR=%{_datadir}/llvm/src \
	-DLLVM_EXTERNAL_LIT=%{_bindir}/lit \
	-DLLVM_LIT_ARGS="-sv \
	--path %{_libdir}/llvm" \
	-DLLVM_LIBDIR_SUFFIX=64

%cmake_build
%cmake_build --target lld-test-depends

%install
head -n -1 %{__cmake_builddir}/test/lit.site.cfg.py >> %{lit_cfg}

# Patch lit config files to load custom config:
for f in %{lit_cfg} ; do
  echo "lit_config.load_config(config, '%{_datadir}/lld/lit.lld-test.cfg.py')" >> $f
done

# Install test files
install -d %{buildroot}%{_datadir}/lld/src
cp %{SOURCE2} %{buildroot}%{_datadir}/lld/

mv %{lit_cfg} %{buildroot}%{_datadir}/lld/src/%{_arch}.site.cfg.py
tar --sort=name --mtime='UTC 2020-01-01' -c test/ | gzip -n > %{buildroot}%{_datadir}/lld/src/test.tar.gz

install -d %{buildroot}%{_libexecdir}/tests/lld
install -m 0755 %{SOURCE1} %{buildroot}%{_libexecdir}/tests/lld
install -d %{buildroot}%{_libdir}/lld/


# Install libraries and binaries
%cmake_install

# This is generated by Patch3000 during build and (probably) must be removed afterward
rm %{buildroot}%{_includedir}/mach-o/compact_unwind_encoding.h

touch %{buildroot}%{_bindir}/ld

install -D -m 644 -t  %{buildroot}%{_mandir}/man1/ docs/ld.lld.1

%post
%{_sbindir}/update-alternatives --install %{_bindir}/ld ld %{_bindir}/ld.lld 1

%postun
if [ $1 -eq 0 ] ; then
  %{_sbindir}/update-alternatives --remove ld %{_bindir}/ld.lld
fi

%check
%cmake_build --target check-lld

%files
%license LICENSE.TXT
%ghost %{_bindir}/ld
%{_bindir}/lld*
%{_bindir}/ld.lld
%{_bindir}/ld64.lld
%{_bindir}/wasm-ld
%{_mandir}/man1/ld.lld.1*

%files devel
%{_includedir}/lld
%{_libdir}/liblld*.so
%{_libdir}/cmake/lld/

%files libs
%{_libdir}/liblld*.so.*

%files test
%{_libexecdir}/tests/lld/
%{_libdir}/lld/
%{_datadir}/lld/src/test.tar.gz
%{_datadir}/lld/src/%{_arch}.site.cfg.py
%{_datadir}/lld/lit.lld-test.cfg.py

%changelog
* Fri May 17 2024 zhanglimin <zhanglimin@loongson.cn> - 17.0.6-2
- Supoort `relax` feature on LoongArch

* Fri Dec 15 2023 Upgrade Robot <upbot@opencloudos.org> - 17.0.6-1
- Upgrade to version 17.0.6

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 16.0.6-2
- Rebuilt for OpenCloudOS Stream 23.09

* Tue Jul 11 2023 kianli <kianli@tencent.com> - 16.0.6-1
- Upgrade to 16.0.6

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 14.0.5-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 14.0.5-2
- Rebuilt for OpenCloudOS Stream 23

* Thu Sep 15 2022 Xiaojie Chen <jackxjchen@tencent.com> - 14.0.5-1
- Initial build
